#version 330 core
layout (location = 0) in vec3 aPos;
layout (location = 1) in vec3 aNormal;
layout (location = 2) in vec2 aTexCoords;

out vec3 FragPos;
out vec2 TexCoords;
out vec3 Normal;

uniform bool invertedNormals;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

uniform bool culling;

const vec4 top = vec4(0, -1, 0, 300);
const vec4 bottom = vec4 (0, 1, 0, 30);
uniform vec4 nearP;

void main()
{
	vec4 worldPos = model * vec4(aPos, 1.0f);

	gl_ClipDistance[0] = culling ? dot(worldPos, top) : 1;
	gl_ClipDistance[1] = culling ? dot(worldPos, bottom) : 1;
	gl_ClipDistance[2] = culling ? dot(worldPos, nearP) : 1;

    vec4 viewPos = view * worldPos;
    FragPos = viewPos.xyz; 
    TexCoords = aTexCoords;
    
    mat3 normalMatrix = transpose(inverse(mat3(view * model)));
    Normal = normalMatrix * (invertedNormals ? -aNormal : aNormal);
    
    gl_Position = projection * viewPos;
}