#version 330 core
layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 textureCoords;

out vec2 TextureCoords;
out vec3 FragmentPosition; 
out vec3 Normal;

out vec4 FragPosLightSpace;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

uniform mat4 lightSpaceMatrix;

uniform bool culling;

const vec4 top = vec4(0, -1, 0, 300);
const vec4 bottom = vec4 (0, 1, 0, 30);
uniform vec4 nearP;

const float shadowDistance = 100.0;
const float transitionDistance = 10.0;

void main() {

	gl_Position = projection * view * model * vec4(position, 1.0f);
    vec4 worldPos = model * vec4(position, 1.0f);

	gl_ClipDistance[0] = culling ? dot(worldPos, top) : 1;
	gl_ClipDistance[1] = culling ? dot(worldPos, bottom) : 1;
	gl_ClipDistance[2] = culling ? dot(worldPos, nearP) : 1;

	FragmentPosition = vec3(worldPos);
	Normal = mat3(transpose(inverse(model))) * normal;
    TextureCoords = textureCoords;

	FragPosLightSpace = lightSpaceMatrix * vec4(FragmentPosition, 1.0);
}